#!/usr/bin/python

import os, sys, traceback, __builtin__
try:
    import readline
except ImportError:
    readline = None

from atop import store

class TransactionShell:
    store = None
    lineBuf = ''

    historyFilename = os.path.expanduser('~/.txnshell_history')

    def __init__(self):
        self.locals = {}
        self.globals = {'__builtins__': __builtin__}

    def run(self):
        if readline is not None:
            try:
                readline.read_history_file(self.historyFilename)
            except IOError:
                pass
        self.running = True
        while self.running:
            try:
                self.iterate()
            except (KeyboardInterrupt, EOFError):
                sys.stdout.write('\n')
                break
        if self.store is not None:
            self.store.close()
        if readline is not None:
            readline.write_history_file(self.historyFilename)

    def iterate(self):
        self.interpretLine(self.readLine())

    def compileSource(self, source):
        try:
            return compile(source, '<string>', 'eval')
        except SyntaxError:
            try:
                return compile(source, '<string>', 'single')
            except SyntaxError, e:
                if 'EOF' in e.args[0]:
                    return None
                raise

    def interpretLine(self, line):
        if line:
            if self.lineBuf:
                self.lineBuf = self.lineBuf + '\n' + line
            else:
                try:
                    code = self.compileSource(line)
                except SyntaxError:
                    self.reportTraceback()
                else:
                    if code is not None:
                        self.runCode(code)
                    else:
                        self.lineBuf = line
        elif self.lineBuf:
            source = self.lineBuf
            self.lineBuf = ''
            try:
                code = self.compileSource(source)
            except SyntaxError:
                self.reportTraceback()
            else:
                if code is not None:
                    self.runCode(code)
                else:
                    self.reportTraceback()

    def runCode(self, code):
        s = self.locals.get('s')
        if isinstance(s, (store.Store, store.SubStore)):
            f = s.transact
            args = eval, code, self.globals, self.locals
        else:
            f = eval
            args = code, self.globals, self.locals
        try:
            result = f(*args)
        except:
            self.reportTraceback()
        else:
            self.locals['_'] = result
            if result is not None:
                sys.stdout.write(repr(result) + '\n')

    def reportTraceback(self):
        traceback.print_exception(file=sys.stdout, *sys.exc_info())

    def readLine(self):
        if self.lineBuf:
            p = '... '
        else:
            p = '>>> '
        return raw_input(p)

def main():
    shell = TransactionShell()
    shell.run()

if __name__ == '__main__':
    main()
