# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

from twisted.trial import unittest
from atop.ifcompat import *

class IAncientInterface(Interface):
    pass

class AncientSuperclass:
    __implements__ = IAncientInterface

class IMyTest1(Interface):
    foo = Attribute("""
    Foo is an attribute, with some documentation.
    """)
    def bar():
        "bar is a method also with some documentation."

class INotInteresting(Interface):
    pass

class ClassicWithStuff:
    implements(IMyTest1)

backwardsCompatImplements(ClassicWithStuff)

class NewStyleWithStuff(object):
    implements(IMyTest1)

backwardsCompatImplements(NewStyleWithStuff)

class ProvideItInline:
    classProvides(IMyTest1)
backwardsCompatImplements(ProvideItInline)

class ProvideItHarder(ProvideItInline):
    classProvides(INotInteresting)
backwardsCompatImplements(ProvideItHarder)

class ExtendsAncient(AncientSuperclass):
    implements(IMyTest1)
backwardsCompatImplements(ExtendsAncient)

def makeProvidesTest(o, exp):
    def test():
        prov = list(providedBy(o))
        prov.sort()
        exp.sort()
        self.assertEquals(prov, exp,
                          "Expected %r to be provided by %r, instead %r was" % (
                exp, obj, prov))
    return test

class BasicTestStuff(unittest.TestCase):

    def checkProvides(self, obj, exp):
        prov = list(providedBy(obj))
        prov.sort()
        exp.sort()
        self.assertEquals(prov, exp,
                          "Expected %r to be provided by %r, instead %r was" % (
                exp, obj, prov))


    def testClassicClassProvidesNothing(self):
        self.checkProvides(ClassicWithStuff, [])
    def testNewStyleClassProvidesNothing(self):
        self.checkProvides(NewStyleWithStuff, [])
    def testClassicInstanceProvidesSomething(self):
        self.checkProvides(
        ClassicWithStuff(), [IMyTest1])
    def testNewStyleInstanceProvidesSomething(self):
        self.checkProvides(
        NewStyleWithStuff(), [IMyTest1])
    def testInlineProvides(self):
        self.checkProvides(
        ProvideItInline, [IMyTest1])
    def testProvidesDoesntPropogate(self):
        self.checkProvides(ProvideItInline(), [])

    def testImplementsAttributeProvides(self):
        self.checkProvides(ExtendsAncient, [])

    def _testImplementsAttributeInstanceProvides(self):
        # This behavior is inconsistent across versions, so it's unsupported.
        self.checkProvides(
            ExtendsAncient(), [IAncientInterface, IMyTest1])

    def testAdaptingSomeStuff(self):
        self.assertIdentical(IMyTest1(ProvideItInline), ProvideItInline)

    def testProvidingInheritance(self):
        dict.fromkeys(providedBy(ProvideItHarder))
        self.assertIdentical(IMyTest1(ProvideItHarder), ProvideItHarder)

    testProvidingInheritance.todo = 'zope.interface is broken'

