
from twisted.python import log

from atop.test import unittest
from atop import store
from atop import sched
from atop import upgrade

def stopReactor(ignored):
    from twisted.internet import reactor
    reactor.crash()

class ReindexTestCase(unittest.TestCase):
    storePowerups = [sched.SystemScheduler]

    def testStringifyUpgradeException(self):
        str(upgrade.QueuedTaskError("It is erroneous"))

    def dbtestSimpleReindex(self):
        p = store.Pool(self.store, "Test Pool")
        items = [store.Item(self.store, str(i).zfill(3)) for i in range(2)]
        map(p.addItem, items)

        p.addIndex(store.NameIndex)
        self.assertEquals(list(p.queryIndex(store.NameIndex)), [])

        d = upgrade.upgradeIndex(p, store.NameIndex)
        from twisted.internet import reactor
        d.addCallback(stopReactor)
        reactor.run()

        self.assertEquals(list(p.queryIndex(store.NameIndex)), items)

    def dbtestSomeOverlap(self):
        p = store.Pool(self.store, "Test Pool")
        items = [store.Item(self.store, str(i).zfill(3)) for i in range(2)]
        map(p.addItem, items)

        p.addIndex(store.NameIndex)
        self.assertEquals(list(p.queryIndex(store.NameIndex)), [])

        newItems = [store.Item(self.store, str(i).zfill(3)) for i in range(2, 4)]
        map(p.addItem, newItems)

        self.assertEquals(list(p.queryIndex(store.NameIndex)), newItems)

        d = upgrade.upgradeIndex(p, store.NameIndex)
        d.addCallback(stopReactor)
        d.addErrback(log.err)
        from twisted.internet import reactor
        reactor.run()

        self.assertEquals(list(p.queryIndex(store.NameIndex)), items + newItems)

from atop.test.test_store import XI, YI, ZI

class CounterTestCase(unittest.TestCase):
    storePowerups = [sched.SystemScheduler]

    # skip = "These hang for buildbot."

    def dbtestSimpleCount(self):
        p = store.Pool(self.store)
        p.addIndex(XI)
        p.addIndex(YI)
        p.addIndex(ZI)

        for idxs in ((XI, YI, ZI), (XI, YI), (XI,)):
            for i in range(3):
                i = store.Item(self.store, str(i))
                for iname in idxs:
                    i.setIndexKey(iname, str(i))
                p.addItem(i)

        for index in p.iterIndexes():
            k = p._formatIndexCountKey(index)
            p.setDataKey(k, '0')

        d = upgrade.recountIndexes(p)
        unittest.deferredResult(d)

        for index, count in (XI, 9), (YI, 6), (ZI, 3), (store.UIDIndex, 9):
            self.assertEquals(p.countIndex(index), count)

class Miscellaneous(unittest.TestCase):
    storePowerups = [sched.SystemScheduler]

    def dbtestEmptyPool(self):
        p = store.Pool(self.store)

        for i in range(3):
            p.addItem(store.Item(self.store))

        unittest.deferredResult(upgrade.emptyPool(p))

        self.assertEquals(list(p.queryIndex()), [])

    def dbtestBackwardsEmptyPool(self):
        p = store.Pool(self.store)

        for i in range(3):
            p.addItem(store.Item(self.store, name=str(i)))

        d = upgrade.emptyPool(p, backwards=True)

        L = []
        for i in range(3, 6):
            L.append(store.Item(self.store, name=str(i)))
            p.addItem(L[-1])

        unittest.deferredResult(d)

        self.assertEquals(list(p.queryIndex()), L)

    def dbtestMoveItems(self, backwards=False):
        p1 = store.Pool(self.store, "A")
        p2 = store.Pool(self.store, "B")

        for i in range(3):
            p1.addItem(store.Item(self.store, str(i)))

        d = upgrade.relocatePoolContents(p1, p2, backwards=backwards)
        unittest.deferredResult(d)

        self.assertEquals(len(list(p1.queryIndex())), 0)
        self.assertEquals(len(list(p2.queryIndex())), 3)

    def dbtestMoveItemsBackwards(self, _impl=dbtestMoveItems):
        return _impl(self, True)
